import java.util.Hashtable;

class SwedishSyntax extends Syntax {
    private static final String vowels = "aeiouy";
    private Hashtable wordStarts;

    SwedishSyntax() {
	super();
	wordStarts = new Hashtable();
    }

    private boolean isVowel(char ch) {
	if (vowels.indexOf(ch) >= 0)
	    return true;
	else
	    return false;
    }

    private boolean isConsonant(char ch) {
	return !isVowel(ch);
    }

    /* If this word has a vowel or not */
    private boolean hasVowel(String word) {
	for (int i=0; i<word.length(); i++)
	    if(isVowel(word.charAt(i)))
		return true;
	return false;
    }

    private boolean hasTripleConsonant(String word) {
	if (word.length() >= 3) {
	    for(int i=0; i < word.length()-2; i++) {
		if (word.charAt(i) == word.charAt(i+1) &&
		    word.charAt(i+1) == word.charAt(i+2))
		    return true;
	    }
	}
	return false;
    }

    private String getLeadingConsonants(String word) {
	if (word.length() < 1)
	    return null;
	int i = 0;
	while(i < word.length() &&
	      isConsonant(word.charAt(i))) {
	    i++;
	}
	if (i > 1) {
	    String beg = word.substring(0,i);
	    return beg;
	} else
	    return null;
    }

    private boolean invalidLeadingConsonants(String word) {
	String lead = getLeadingConsonants(word);
	String old = null;
	if (lead == null)
	    return false;
	// Look it up in the hash table to see if it is OK.
	try {
	    old = (String) wordStarts.get(lead);
	} catch (NullPointerException np) {}
	if (old != null) {
	    // If it WAS in the table, it is OK.
	    return false;
	}
	// It was invalid
	return true;
    }

    private boolean okSwedishSyntax(String word) {
	// A word must have a vowel
	if (!hasVowel(word)) {
	    return false;
	}
	// Only a few words of length 1 exist!
	if (word.length() == 1) {
	    if (word.charAt(0) != 'i' && 
		word.charAt(0) != '' && 
		word.charAt(0) != '' &&
		word.charAt(0) != '')
		return false;
	}
	// Words of length 2 only allowed if the letters differ
	else if (word.length() == 2) {
	    if (word.charAt(0) == word.charAt(1))
		return false;
	}
	// Words of length 3 only allowed if the letters differ
	// in each position.
	else if (word.length() == 3) {
	    if (word.charAt(0) == word.charAt(1) ||
		word.charAt(1) == word.charAt(2))
		return false;
	    // For 3-letter words, if the first char is a consonant
	    // the second HAVE to be a vowel
	    else if (isConsonant(word.charAt(0)) &&
		     !isVowel(word.charAt(1)))
		return false;
	}

	// If first two chars are consonants, the third MUST be a vowel
	if (word.length() >= 3) {
	    if (isConsonant(word.charAt(0)) &&
		isConsonant(word.charAt(1)) &&
		!isVowel(word.charAt(2))) {
		return false;
	    }
	    // Inga dubbeltecknade konsonanter i brjan
	    // av ord.
	    if (isConsonant(word.charAt(0)) &&
		isConsonant(word.charAt(1)) &&
		word.charAt(0) == word.charAt(1)) {
		return false;
	    }
	}
    
	// And if first char is a vowel, the second MUST be a consonant
	if (word.length() >= 2 &&
	    isVowel(word.charAt(0)) &&
	    !isConsonant(word.charAt(1))) {
	    return false;
	}

	// Test for excesses in successive consonants
	if (hasTripleConsonant(word))
	    return false;

	if (invalidLeadingConsonants(word))
	    return false;

	return true;

    }

    /* Tests the word according to some grammar
     * rules etc, to remove invalid words.
     */
    public boolean testWord(String word) {
	return okSwedishSyntax(word);
    }

    /* Use this word to build a validity table
     * with some characteristics of allowed words 
     */
    public void consumeWord(String word) {
	// Test for valid consonant sequence at the start
	// of a word
	String beg = getLeadingConsonants(word);
	if (beg != null) {
	    String old = null;
	    try {
		old = (String) wordStarts.get(beg);
	    } catch (NullPointerException np) {}
	    if (old == null) {
		wordStarts.put(beg, beg);
		// Uncomment to dump out valid beginnings as we work
		// System.out.println(beg);
	    }
	}
    }
}
